/***************************************************************************
 *   Copyright (C) 2009 by Stefan Bühler                                   *
 *   stbuehler@web.de                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef TORUSCHESSTESTGAME_H
#define TORUSCHESSTESTGAME_H

#include <QWidget>
#include <QSvgRenderer>
#include <QMainWindow>
#include <QSize>

#include "toruschess.h"

/**
	@author Stefan Bühler <stbuehler@web.de>
*/
namespace toruschess {
	
	class PieceLibrary : public QObject {
	Q_OBJECT
	public:
		PieceLibrary(QObject *parent);
		virtual ~PieceLibrary();
		
		QSize pieceSize() const { return QSize(m_width, m_height); }
		QSize minPieceSize() const { return QSize(20, 20); }
		void setSize(int width, int height);
		
		void paint(QPainter &pt, int piece) const;
		
	private:
		QSvgRenderer **m_pieces;
		QImage **m_buffers;
		int m_width, m_height;
	};
	
	class TestPlace : public QWidget {
	Q_OBJECT
	public:
		TestPlace(QWidget *parent, const PieceLibrary *lib, const Field *field, const Pos &p);
	
		const Pos& pos() const { return m_pos; }
		const Field* field() const { return m_field; }
	
		virtual QSize sizeHint() const;
		
	protected:
		virtual void paintEvent(QPaintEvent *event);
	
	private:
		const Field *m_field;
		const PieceLibrary *m_lib;
		Pos m_pos;
	};
	
	class TestField : public QWidget {
	Q_OBJECT
	public:
		TestField(QWidget *parent, PieceLibrary *lib, Game *m_game);
	
	protected:
		virtual void resizeEvent(QResizeEvent *event);
	
	private:
		PieceLibrary *m_lib;
		Game *m_game;
	};
	
	class TestGame : public QMainWindow {
	Q_OBJECT
	public:
		TestGame(QWidget *parent = 0);
	
	private:
		Game *m_game;
	};
}

#endif
